@@ -20,6 +20,9 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include <glade/glade.h>
 #include <gal/widgets/widget-color-combo.h>
 
@@ -33,10 +36,10 @@
 #include "htmlengine-edit-tablecell.h"
 #include "htmlengine-save.h"
 #include "htmlimage.h"
+#include "htmltable.h"
 #include "htmltablecell.h"
 #include "htmlsettings.h"
 
-#include "config.h"
 #include "properties.h"
 #include "cell.h"
 #include "utils.h"
@@ -64,7 +67,7 @@ typedef struct
 
 	gboolean   has_bg_pixmap;
 	gboolean   changed_bg_pixmap;
-	gchar     *bg_pixmap;
+	const gchar *bg_pixmap;
 	GtkWidget *entry_bg_pixmap;
 	GtkWidget *check_bg_pixmap;
 
@@ -92,6 +95,11 @@ typedef struct
 	GtkWidget *check_height;
 	GtkWidget *option_height;
 
+	gint cspan;
+	gint rspan;
+	GtkWidget *spin_cspan;
+	GtkWidget *spin_rspan;
+
 	gboolean   wrap;
 	gboolean   changed_wrap;
 	GtkWidget *option_wrap;
@@ -114,7 +122,7 @@ static void
 fill_sample (GtkHTMLEditCellProperties *d)
 {
 	GString *str;
-	gchar *body, *bg_color, *bg_pixmap, *valign, *halign, *width, *height, *wrap, *cell;
+	gchar *body, *bg_color, *bg_pixmap, *valign, *halign, *width, *height, *wrap, *cell, *cspan, *rspan;
 	gint r, c;
 
 	body      = html_engine_save_get_sample_body (d->cd->html->engine, NULL);
@@ -142,15 +150,17 @@ fill_sample (GtkHTMLEditCellProperties *d)
 		? g_strdup_printf (" height=\"%d%s\"", d->height, d->height_percent ? "%" : "") : g_strdup ("");
 	wrap    = d->wrap ? " nowrap" : "";
 
+	cspan   = d->cspan > 1 ? g_strdup_printf (" colspan=%d", d->cspan) : g_strdup ("");
+	rspan   = d->rspan > 1 ? g_strdup_printf (" rowspan=%d", d->rspan) : g_strdup ("");
 	cell    = g_strconcat ("<", d->heading ? "th" : "td",
-			       bg_color, bg_pixmap, halign, valign, width, height, wrap, ">", NULL);
+			       bg_color, bg_pixmap, halign, valign, width, height, cspan, rspan, wrap, ">", NULL);
 
 	str = g_string_new (body);
 	g_string_append (str, "<table border=1 cellpadding=4 cellspacing=2>");
 
-	for (r = 0; r < 2; r ++) {
+	for (r = 0; r < d->rspan + 1; r ++) {
 		g_string_append (str, "<tr>");
-		for (c = 0; c < 3; c ++) {
+		for (c = 0; c < (r < d->rspan ? (r == 0 ? 3 : 2) : d->cspan + 2); c ++) {
 			if ((r == 0 && c == 1)
 			    || (d->scope == ROW && r == 0)
 			    || (d->scope == COLUMN && c == 1)
@@ -159,10 +169,16 @@ fill_sample (GtkHTMLEditCellProperties *d)
 			else
 				g_string_append (str, "<td>");
 
-			g_string_append (str, r == 0 && c == 1
-					 ? "The quick brown fox jumps over the lazy dog. "
-					 "The quick brown fox jumps over the lazy dog."
-					 : "&nbsp;Other&nbsp;");
+			if (c == 1 && r == 0) {
+				/* string marked for translations is sample text. you may want to translate it to common sample text in your language */
+				g_string_append (str, _("The quick brown fox jumps over the lazy dog."));
+				g_string_append (str, " ");
+				g_string_append (str, _("The quick brown fox jumps over the lazy dog."));
+			} else {
+				g_string_append (str, "&nbsp;");
+				g_string_append (str, _("Other"));
+				g_string_append (str, "&nbsp;");
+			}
 			g_string_append (str, "</td>");
 		}
 		g_string_append (str, "</tr>");
@@ -176,6 +192,8 @@ fill_sample (GtkHTMLEditCellProperties *d)
 	g_free (bg_pixmap);
 	g_free (body);
 	g_free (cell);
+	g_free (rspan);
+	g_free (cspan);
 	g_string_free (str, TRUE);
 }
 
@@ -216,7 +234,7 @@ set_has_bg_pixmap (GtkWidget *check, GtkHTMLEditCellProperties *d)
 }
 
 static void
-changed_bg_color (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditCellProperties *d)
+changed_bg_color (GtkWidget *w, GdkColor *color, gboolean custom, gboolean by_user, gboolean is_default, GtkHTMLEditCellProperties *d)
 {
 	/* If the color was changed programatically there's not need to set things */
 	if (!by_user)
@@ -271,6 +289,22 @@ changed_valign (GtkWidget *w, GtkHTMLEditCellProperties *d)
 	CHANGE;	
 }
 
+static void
+changed_cspan (GtkWidget *w, GtkHTMLEditCellProperties *d)
+{
+	d->cspan = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (d->spin_cspan));
+	FILL;
+	CHANGE;
+}
+
+static void
+changed_rspan (GtkWidget *w, GtkHTMLEditCellProperties *d)
+{
+	d->rspan = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (d->spin_rspan));
+	FILL;
+	CHANGE;
+}
+
 static void
 changed_width (GtkWidget *w, GtkHTMLEditCellProperties *d)
 {
@@ -380,7 +414,7 @@ cell_widget (GtkHTMLEditCellProperties *d)
 	GtkWidget *cell_page;
 	GladeXML *xml;
 
-	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "cell_page");
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "cell_page", NULL);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -390,63 +424,62 @@ cell_widget (GtkHTMLEditCellProperties *d)
 	html_color_alloc (color, d->cd->html->engine->painter);
 	d->combo_bg_color = color_combo_new (NULL, _("Automatic"), &color->color,
 					     color_group_fetch ("cell_bg_color", d->cd));
-        gtk_signal_connect (GTK_OBJECT (d->combo_bg_color), "changed", GTK_SIGNAL_FUNC (changed_bg_color), d);
+        g_signal_connect (d->combo_bg_color, "color_changed", G_CALLBACK (changed_bg_color), d);
 	gtk_table_attach (GTK_TABLE (glade_xml_get_widget (xml, "table_cell_bg")),
 			  d->combo_bg_color,
 			  1, 2, 0, 1, 0, 0, 0, 0);
 
 	d->check_bg_color  = glade_xml_get_widget (xml, "check_cell_bg_color");
-	gtk_signal_connect (GTK_OBJECT (d->check_bg_color), "toggled", set_has_bg_color, d);
+	g_signal_connect (d->check_bg_color, "toggled", G_CALLBACK (set_has_bg_color), d);
 	d->check_bg_pixmap = glade_xml_get_widget (xml, "check_cell_bg_pixmap");
-	gtk_signal_connect (GTK_OBJECT (d->check_bg_pixmap), "toggled", set_has_bg_pixmap, d);
+	g_signal_connect (d->check_bg_pixmap, "toggled", G_CALLBACK (set_has_bg_pixmap), d);
 	d->entry_bg_pixmap = glade_xml_get_widget (xml, "entry_cell_bg_pixmap");
 
-	d->disable_change = TRUE;
-	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
-	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))));
-	our_gnome_pixmap_entry_set_last_dir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap));
-	d->disable_change = FALSE;
-
-	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
-			    "changed", GTK_SIGNAL_FUNC (changed_bg_pixmap), d);
+	g_signal_connect (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->entry_bg_pixmap)),
+			    "changed", G_CALLBACK (changed_bg_pixmap), d);
 
 	d->option_halign = glade_xml_get_widget (xml, "option_cell_halign");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_halign))), "selection-done",
-			    changed_halign, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_halign)), "selection-done",
+			  G_CALLBACK (changed_halign), d);
 	d->option_valign = glade_xml_get_widget (xml, "option_cell_valign");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_valign))), "selection-done",
-			    changed_valign, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_valign)), "selection-done",
+			  G_CALLBACK (changed_valign), d);
 
 	d->spin_width   = glade_xml_get_widget (xml, "spin_cell_width");
 	UPPER_FIX (width);
-	gtk_signal_connect (GTK_OBJECT (d->spin_width), "changed", changed_width, d);
+	g_signal_connect (d->spin_width, "value_changed", G_CALLBACK (changed_width), d);
 	d->check_width  = glade_xml_get_widget (xml, "check_cell_width");
-	gtk_signal_connect (GTK_OBJECT (d->check_width), "toggled", set_has_width, d);
+	g_signal_connect (d->check_width, "toggled", G_CALLBACK (set_has_width), d);
 	d->option_width = glade_xml_get_widget (xml, "option_cell_width");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width))), "selection-done",
-			    changed_width_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width)), "selection-done",
+			  G_CALLBACK (changed_width_percent), d);
 
 	d->spin_height   = glade_xml_get_widget (xml, "spin_cell_height");
 	UPPER_FIX (height);
-	gtk_signal_connect (GTK_OBJECT (d->spin_height), "changed", changed_height, d);
+	g_signal_connect (d->spin_height, "value_changed", G_CALLBACK (changed_height), d);
 	d->check_height  = glade_xml_get_widget (xml, "check_cell_height");
-	gtk_signal_connect (GTK_OBJECT (d->check_height), "toggled", set_has_height, d);
+	g_signal_connect (d->check_height, "toggled", G_CALLBACK (set_has_height), d);
 	d->option_height = glade_xml_get_widget (xml, "option_cell_height");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_height))), "selection-done",
-			    changed_height_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_height)), "selection-done",
+			  G_CALLBACK (changed_height_percent), d);
 
 	d->option_wrap    = glade_xml_get_widget (xml, "option_cell_wrap");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_wrap))), "selection-done",
-			    changed_wrap, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_wrap)), "selection-done",
+			  G_CALLBACK (changed_wrap), d);
 	d->option_heading = glade_xml_get_widget (xml, "option_cell_style");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_heading))), "selection-done",
-			    changed_heading, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_heading)), "selection-done",
+			  G_CALLBACK (changed_heading), d);
 
 	d->option_scope   = glade_xml_get_widget (xml, "option_cell_scope");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_scope))), "selection-done",
-			    changed_scope, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_scope)), "selection-done",
+			  G_CALLBACK (changed_scope), d);
 
-	gtk_box_pack_start (GTK_BOX (cell_page), sample_frame (&d->sample), FALSE, FALSE, 0);
+	d->spin_cspan   = glade_xml_get_widget (xml, "spin_cell_cspan");
+	d->spin_rspan   = glade_xml_get_widget (xml, "spin_cell_rspan");
+	g_signal_connect (d->spin_cspan, "value_changed", G_CALLBACK (changed_cspan), d);
+	g_signal_connect (d->spin_rspan, "value_changed", G_CALLBACK (changed_rspan), d);
+
+	gtk_box_pack_start_defaults (GTK_BOX (cell_page), sample_frame (&d->sample));
 
 	gtk_widget_show_all (cell_page);
 	gnome_pixmap_entry_set_preview (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap), FALSE);
@@ -460,11 +493,11 @@ set_ui (GtkHTMLEditCellProperties *d)
 	d->disable_change = TRUE;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (d->check_bg_color), d->has_bg_color);
-	gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &d->bg_color);
+	/* RM2 gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &d->bg_color); */
 	color_combo_set_color (COLOR_COMBO (d->combo_bg_color), &d->bg_color);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (d->check_bg_pixmap), d->has_bg_pixmap);
-	gtk_entry_set_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
+	gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->entry_bg_pixmap))),
 			    d->bg_pixmap);
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_halign), d->halign - HTML_HALIGN_LEFT);
@@ -481,9 +514,8 @@ set_ui (GtkHTMLEditCellProperties *d)
 	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_wrap), d->wrap ? 1 : 0);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_heading), d->heading ? 1 : 0);
 
-	/* gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_spacing), d->spacing);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_padding), d->padding);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_border),  d->border); */
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_cspan),  d->cspan);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_rspan),  d->rspan);
 
 	d->disable_change = FALSE;
 
@@ -524,22 +556,8 @@ get_data (GtkHTMLEditCellProperties *d)
 		d->has_width = TRUE;
 	}
 
-	/* d->spacing = d->table->spacing;
-	d->padding = d->table->padding;
-	d->border  = d->table->border;
-
-	g_return_if_fail (HTML_OBJECT (d->table)->parent);
-	d->align   = HTML_CLUE (HTML_OBJECT (d->table)->parent)->halign;
-
-	if (HTML_OBJECT (d->table)->percent) {
-		d->width = HTML_OBJECT (d->table)->percent;
-		d->width_percent = TRUE;
-		d->has_width = TRUE;
-	} else if (d->table->specified_width) {
-		d->width = d->table->specified_width;
-		d->width_percent = FALSE;
-		d->has_width = TRUE;
-		} */
+	d->cspan = d->cell->cspan;
+	d->rspan = d->cell->rspan;
 }
 
 GtkWidget *
@@ -584,6 +602,8 @@ cell_apply_1 (HTMLTableCell *cell, GtkHTMLEditCellProperties *d)
 	if (d->changed_width)
 		html_engine_table_cell_set_width (d->cd->html->engine, cell,
 						  d->has_width ? d->width : 0, d->has_width ? d->width_percent : FALSE);
+	html_engine_set_cspan (d->cd->html->engine, d->cspan);
+	html_engine_set_rspan (d->cd->html->engine, d->rspan);
 }
 
 static void
@@ -592,7 +612,7 @@ cell_apply_row (GtkHTMLEditCellProperties *d)
 	HTMLTableCell *cell;
 	HTMLEngine *e = d->cd->html->engine;
 
-	if (html_engine_table_goto_row (e, d->cell->row)) {
+	if (html_engine_table_goto_row (e, HTML_TABLE (HTML_OBJECT (d->cell)->parent), d->cell->row)) {
 		cell = html_engine_get_table_cell (e);
 
 		while (cell && cell->row == d->cell->row) {
@@ -610,7 +630,7 @@ cell_apply_col (GtkHTMLEditCellProperties *d)
 	HTMLTableCell *cell;
 	HTMLEngine *e = d->cd->html->engine;
 
-	if (html_engine_table_goto_col (e, d->cell->col)) {
+	if (html_engine_table_goto_col (e, HTML_TABLE (HTML_OBJECT (d->cell)->parent), d->cell->col)) {
 		cell = html_engine_get_table_cell (e);
 
 		while (cell) {
@@ -625,10 +645,12 @@ cell_apply_col (GtkHTMLEditCellProperties *d)
 static void
 cell_apply_table (GtkHTMLEditCellProperties *d)
 {
+	HTMLTable *table;
 	HTMLTableCell *cell;
 	HTMLEngine *e = d->cd->html->engine;
 
-	if (html_engine_table_goto_0_0 (e)) {
+	table = html_engine_get_table (e);
+	if (table && html_engine_goto_table_0 (e, table)) {
 		cell = html_engine_get_table_cell (e);
 
 		while (cell) {
@@ -640,7 +662,7 @@ cell_apply_table (GtkHTMLEditCellProperties *d)
 	}
 }
 
-void
+gboolean
 cell_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditCellProperties *d = (GtkHTMLEditCellProperties *) get_data;
@@ -649,6 +671,21 @@ cell_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 
 	position = e->cursor->position;
 
+	if (html_engine_get_table_cell (e) != d->cell) {
+		if (!html_engine_goto_table (e, HTML_TABLE (HTML_OBJECT (d->cell)->parent), d->cell->row, d->cell->col)) {
+			GtkWidget *dialog;
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (d->cd->properties_dialog->dialog),
+							 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+							 _("The editted cell was removed from the document.\nCannot apply your changes."));
+			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			html_cursor_jump_to_position (e->cursor, e, position);
+
+			return FALSE;
+		}
+	}
+
 	switch (d->scope) {
 	case CELL:
 		cell_apply_1 (d->cell, d);
@@ -664,13 +701,7 @@ cell_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 	}
 	html_cursor_jump_to_position (e->cursor, e, position);
 
-	d->changed_bg_color = FALSE;
-	d->changed_bg_pixmap = FALSE;
-	d->changed_halign = FALSE;
-	d->changed_valign = FALSE;
-	d->changed_wrap = FALSE;
-	d->changed_heading = FALSE;
-	d->changed_width = FALSE;
+	return TRUE;
 }
 
 void
