@@ -25,14 +25,15 @@
 
 #include "control-data.h"
 
-typedef GnomeDialog ** (*DialogCtor)(GtkHTML *html);
+typedef GtkDialog ** (*DialogCtor)(GtkHTML *html, GtkHTMLControlData *cd);
 
-#define RUN_DIALOG(name,title) run_dialog ((GnomeDialog ***)&cd-> name ## _dialog, cd->html, (DialogCtor) gtk_html_ ## name ## _dialog_new, title)
+#define RUN_DIALOG(name,title) run_dialog ((GtkDialog ***)&cd-> name ## _dialog, cd->html, cd, (DialogCtor) gtk_html_ ## name ## _dialog_new, title)
 
-void       run_dialog         (GnomeDialog ***dialog,
-			       GtkHTML       *html,
-			       DialogCtor     ctor,
-			       const gchar   *title);
-GtkWindow *get_parent_window  (GtkWidget     *w);
+void       run_dialog         (GtkDialog          ***dialog,
+			       GtkHTML              *html,
+			       GtkHTMLControlData   *cd,
+			       DialogCtor            ctor,
+			       const gchar          *title);
+GtkWindow *get_parent_window  (GtkWidget            *w);
 
 #endif
