@@ -73,10 +73,10 @@ ask_dialog_new (HTMLEngine *e)
 
 	d = g_new (GtkHTMLReplaceAskDialog, 1);
 	d->dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Replace confirmation"), NULL, 0,
-							     _("Replace _All"), 1,
-							     _("_Next"), 2,
+							     _("Replace all"), 1,
+							     _("Next"), 2,
 							     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-							     _("_Replace"), 0,
+							     _("Replace"), 0,
 							     NULL));
 	d->engine = e;
 
@@ -94,6 +94,7 @@ ask (HTMLEngine *e, gpointer data)
 	ask_dialog = ask_dialog_new (e);
 	gtk_dialog_run (ask_dialog->dialog);
 	gtk_widget_destroy (GTK_WIDGET (ask_dialog->dialog));
+	gtk_widget_grab_focus (GTK_WIDGET (ask_dialog->engine->widget));
 	g_free (ask_dialog);
 }
 
@@ -132,6 +133,9 @@ replace_dialog_response (GtkDialog *dialog, gint response_id, GtkHTMLReplaceDial
 				     GTK_TOGGLE_BUTTON (d->backward)->active == 0, FALSE,
 				     ask, d);
 		break;
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_grab_focus (GTK_WIDGET (d->html));
+		break;
 	}
 }
 
