@@ -25,6 +25,7 @@
 #include "htmlbutton.h"
 #include "htmlform.h"
 #include <string.h>
+#include <libgnome/gnome-i18n.h>
 
 HTMLButtonClass html_button_class;
 
@@ -130,18 +131,18 @@ html_button_init (HTMLButton *button,
 
 	html_embedded_init (element, HTML_EMBEDDED_CLASS (klass), parent, name, value);
 	
-	if( strlen (element->value))
+	if( strlen (element->value)) {
 		widget = gtk_button_new_with_label (element->value);
-	else {
+	} else {
 		switch(type) {
 		case BUTTON_NORMAL:
 			widget = gtk_button_new ();
 			break;
 		case BUTTON_SUBMIT:
-			widget = gtk_button_new_with_label ("Submit Query");
+			widget = gtk_button_new_with_label (_("Submit Query"));
 			break;
 		case BUTTON_RESET:
-			widget = gtk_button_new_with_label ("Reset");
+			widget = gtk_button_new_with_label (_("Reset"));
 			break;
 		default:
 			g_assert_not_reached ();
@@ -150,8 +151,7 @@ html_button_init (HTMLButton *button,
 
 	html_embedded_set_widget (element, widget);
 
-	gtk_signal_connect (GTK_OBJECT (widget), "clicked",
-                            GTK_SIGNAL_FUNC (clicked_event), button);
+	g_signal_connect (widget, "clicked", G_CALLBACK (clicked_event), button);
 
 	button->type = type;
 	button->successful = FALSE;
