@@ -44,7 +44,6 @@
 #include "htmlengine-edit-selection-updater.h"
 #include "htmlimage.h"
 #include "htmlinterval.h"
-#include "htmllinktext.h"
 #include "htmlobject.h"
 #include "htmlplainpainter.h"
 #include "htmltable.h"
@@ -876,8 +875,7 @@ insert_object_do (HTMLEngine *e, HTMLObject *obj, guint *len, gint level, guint
 	printf ("position after merge %d\n", e->cursor->position);
 #endif
 
-	html_cursor_destroy (e->cursor);
-	e->cursor = html_cursor_dup (orig);
+	html_cursor_copy (e->cursor, orig);
 	html_cursor_jump_to_position_no_spell (e->cursor, e, position_after);
 
 	if (check)
@@ -1145,7 +1143,7 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 	remove_empty_and_merge (e, FALSE, left, right, orig);
 
 	/* replace empty link in empty flow by text with the same style */
-	if (HTML_IS_LINK_TEXT (e->cursor->object) && html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
+	/* FIXME-link if (HTML_IS_LINK_TEXT (e->cursor->object) && html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
 		HTMLObject *flow = e->cursor->object->parent;
 		HTMLObject *new_text;
 
@@ -1160,7 +1158,7 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 			orig->object = e->cursor->object;
 		}
 		html_clue_append (HTML_CLUE (flow), e->cursor->object);
-	}
+		} */
 
 	html_cursor_forward (e->cursor, e);
 
@@ -1200,8 +1198,8 @@ html_engine_insert_empty_paragraph (HTMLEngine *e)
 	insert_empty_paragraph (e, HTML_UNDO_UNDO, TRUE);
 }
 
-static char *picto_chars = "DO)(|/P\0:-\0:\0:-\0:\0:;=-\0:;\0:-~\0:\0:\0:-\0:\0:-\0:\0:-\0:\0";
-static gint picto_states [] = { 8, 13, 18, 26, 34, 39, 44, 0, -1, 11, 0, -1, 0, -2, 16, 0, -2, 0, -3, -4, -5, 23, 0, -3, -4, 0, -6, 30, 32, 0, -6, 0, -11, 0, -8, 37, 0, -8, 0, -9, 42, 0, -9, 0, -10, 47, 0, -10, 0};
+static char *picto_chars = "DO)(|/PQ\0:-\0:\0:-\0:\0:;=-\0:;\0:-~\0:\0:\0:-\0:\0:-\0:\0:-\0:\0:-\0:\0";
+static gint picto_states [] = { 9, 14, 19, 27, 35, 40, 45, 50, 0, -1, 12, 0, -1, 0, -2, 17, 0, -2, 0, -3, -4, -5, 24, 0, -3, -4, 0, -6, 31, 33, 0, -6, 0, -11, 0, -8, 38, 0, -8, 0, -9, 43, 0, -9, 0, -10, 48, 0, -10, 0, -12, 53, 0, -12, 0};
 static gchar *picto_images [] = {
 	"smiley-1.png",
 	"smiley-2.png",
@@ -1214,6 +1212,7 @@ static gchar *picto_images [] = {
 	"smiley-9.png",
 	"smiley-10.png",
 	"smiley-11.png",
+	"smiley-12.png",
 };
 
 static void
@@ -1272,7 +1271,7 @@ use_pictograms (HTMLEngine *e)
 }
 
 void
-html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
+html_engine_insert_text_with_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
 {
 	gchar *nl;
 	gint alen;
@@ -1300,6 +1299,10 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 				html_engine_set_insertion_link (e, NULL, NULL);
 
 			o = html_engine_new_text (e, text, alen);
+			if (attrs) {
+				pango_attr_list_splice (HTML_TEXT (o)->attr_list, attrs, 0, HTML_TEXT (o)->text_len);
+				gtk_html_debug_list_text_attrs (HTML_TEXT (o));
+			}
 			html_text_convert_nbsp (HTML_TEXT (o), TRUE);
 
 			if (alen == 1 && html_is_in_word (html_text_get_char (HTML_TEXT (o), 0))
@@ -1324,7 +1327,13 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 }
 
 void
-html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
+html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
+{
+	html_engine_insert_text_with_attributes (e, text, len, NULL);
+}
+
+void
+html_engine_paste_text_with_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
 {
 	gchar *undo_name = g_strdup_printf ("Paste text: '%s'", text);
 	gchar *redo_name = g_strdup_printf ("Unpaste text: '%s'", text);
@@ -1333,10 +1342,16 @@ html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
 	g_free (undo_name);
 	g_free (redo_name);
 	html_engine_delete (e);
-	html_engine_insert_text (e, text, len);
+	html_engine_insert_text_with_attributes (e, text, len, attrs);
 	html_undo_level_end (e->undo);
 }
 
+void
+html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
+{
+	html_engine_paste_text_with_attributes (e, text, len, NULL);
+}
+
 void
 html_engine_delete_container (HTMLEngine *e)
 {
