@@ -35,7 +35,6 @@
 #include "htmlclueflow.h"
 #include "htmlcolorset.h"
 #include "htmlcursor.h"
-#include "htmllinktext.h"
 #include "htmlobject.h"
 #include "htmltable.h"
 #include "htmltext.h"
@@ -92,7 +91,7 @@ html_engine_set_mark (HTMLEngine *e)
 {
 	g_return_if_fail (e != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (e));
-	g_return_if_fail (e->editable);
+	g_return_if_fail (e->editable || e->caret_mode);
 
 	if (e->mark != NULL)
 		html_engine_unselect_all (e);
@@ -336,11 +335,19 @@ html_engine_clipboard_clear (HTMLEngine *e)
 HTMLObject *
 html_engine_new_text (HTMLEngine *e, const gchar *text, gint len)
 {
-	if (e->insertion_url && *e->insertion_url) {
-		return html_link_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color,
-						    e->insertion_url, e->insertion_target);
-	} else
-		return html_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color);
+	HTMLObject *to;
+
+	to = html_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color);
+	if (e->insertion_font_style != GTK_HTML_FONT_STYLE_DEFAULT)
+		html_text_set_style_in_range (HTML_TEXT (to), e->insertion_font_style, e, 0, HTML_TEXT (to)->text_bytes);
+	if (e->insertion_color &&
+	    ((e->insertion_url == NULL && e->insertion_color != html_colorset_get_color (e->settings->color_set, HTMLTextColor))
+	     || (e->insertion_url && e->insertion_color != html_colorset_get_color (e->settings->color_set, HTMLLinkColor))))
+		html_text_set_color_in_range (HTML_TEXT (to), e->insertion_color, 0, HTML_TEXT (to)->text_bytes);
+	if (e->insertion_url)
+		html_text_append_link (HTML_TEXT (to), e->insertion_url, e->insertion_target, 0, HTML_TEXT (to)->text_len);
+
+	return to;
 }
 
 HTMLObject *
@@ -356,9 +363,9 @@ html_engine_new_link (HTMLEngine *e, const gchar *text, gint len, gchar *url)
 	} else
 		real_url = url;
 		
-	link = html_link_text_new_with_len (text, len, e->insertion_font_style,
-					    html_colorset_get_color (e->settings->color_set, HTMLLinkColor),
-					    real_url, real_target);
+	link = html_text_new_with_len (text, len, e->insertion_font_style,
+				       html_colorset_get_color (e->settings->color_set, HTMLLinkColor));
+	html_text_append_link (HTML_TEXT (link), real_url, real_target, 0, HTML_TEXT (link)->text_len);
 
 	if (real_target)
 		g_free (real_url);
